
class CatalogWrapper:
    
    """
    Wraps the catalog so that it's easier to return the results we are
    interrested in.
    """
    
    def __init__(self, catalog, return_objects=0):
        self._catalog = catalog
        # returns filtered objects instead of brains if true
        self.return_objects = return_objects


    def getTypesInRange(self, first_date, last_date, **kw):
        "Search catalog for events for all attendees"
        catalog = self._catalog
        # Get all events that starts inside range
        brains = catalog(
            start={'query': (first_date, last_date), 'range': 'min:max'},
            **kw
        )
        # Get all events that ends inside range
        brains += catalog(
            end={'query': (first_date, last_date), 'range': 'min:max'},
            **kw
        )
        # Get all events that Start before this day AND End after this day
        brains += catalog(
            start={'query': first_date, 'range': 'max'},
            end={'query': last_date, 'range': 'min'},
            **kw
        )
        brains = self.uniqueBrains(brains)
        if self.return_objects:
            return self.brains2objects(brains)
        else:
            return brains


    def searchResults(self, REQUEST=None, **kw):
        "The main catalog wrapper"
        brains = self._catalog.searchResults(REQUEST, **kw)
        if self.return_objects:
            return self.brains2objects(brains)
        else:
            return brains

    __call__ = searchResults


    def uniqueBrains(self, brains):
        "Removes duplicate brains from list"
        unique = {}
        for brain in brains:
            unique[brain.getRID()] = brain
        return unique.values()       


    def brains2objects(self, brains):
        "Safely returns a list of objects from list of brains"
        result = []
        for brain in brains:
            try:
                obj = brain.getObject()
                if not obj is None:
                    result.append(obj)
            except:
                pass
        return result
