class EventMover:

    """
    A wrapper for moving events. Events must have getStart, getEnd, setStart & 
    setEnd methods.
    """

    def __init__(self, obj):
        self._obj = obj

    def getStart(self):
        return self._obj.getStart()

    def getEnd(self):
        return self._obj.getEnd()

    def setStart(self, start):
        self._obj.setStart(start)
        self._obj._p_changed = 1

    def setEnd(self, end):
        self._obj.setEnd(end)
        self._obj._p_changed = 1

    def getDuration(self):
        "Returns the duration or None"
        start = self.getStart()
        end = self.getEnd()
        if start and end:
            return end - start

    def setDuration(self, duration):
        "Sets the duration by setting the end time relative to the start time"
        self.setEnd(self.getStart() + duration)

    def getDelta(self, new_start):
        "Returns the delta between old an new start"
        start = self.getStart()
        if start:
            return new_start - start

    def moveAbsolute(self, new_start):
        "Moves the event to new absolute start time, keeping duration"
        duration = self.getDuration()
        if duration:
            self.setStart(new_start)
            self.setDuration(duration)

    def moveRelative(self, delta):
        """
        Moves the event to new start time relative to current position,
        keeping duration
        """
        start = self.getStart()
        if start:
            self.setStart(start + delta)
        end = self.getEnd()
        if end:
            self.setEnd(end + delta)

if __name__ == '__main__':

    class Event:

        "simple class for testing"

        def __init__(self, start, end):
            self.setStart(start)
            self.setEnd(end)

        def getStart(self):
            return self.start

        def getEnd(self):
            return self.end

        def setStart(self, start):
            self.start = start

        def setEnd(self, end):
            self.end = end

        def __str__(self):
            return "start: %s, end: %s" % (self.start, self.end)

    ev = Event(14, 20)
    mv = EventMover(ev)
    mv.moveRelative(5)
    print ev
    mv.moveAbsolute(0)
    print ev
