# -*- coding: ISO-8859-1 -*-

from AccessControl import ClassSecurityInfo
from random import randint
from DateTime import DateTime
from TrackCalc import TrackCalc

"""
This is an example "interface" for a gantadapter.
"""

########################################
# Example of the dictionary structure
tracks = []
for i in range(20) : 
    tracks.append({
        'title' : 'Track #%s' % i,
        'absolute_url' : '/track?no=%s' % i,
        'start' : randint(0,10),
        'duration' : randint(0,80),
        'startdate' : DateTime(2005, 8, 1),
        'enddate' : DateTime(2005, 8, 8),
        'title' : 'Projekt #%s' % i,
        'show_begin' : 1,
        'show_duration' : 1,
        'show_end' : 1,
    })

daterange = []
date = DateTime()
for i in range(1,80):
    daterange.append(date)
    date += 1

adapter = {
    'date_format_short':'%d/%m/%Y',
    'date_format_long':'%d/%m/%Y %H:%S',
    'daterange' : daterange,
    'start_date': daterange[0],
    'end_date' : daterange[-1],
    'portal_startyear':2001,
    'portal_endyear':2010,
    'day_size' : 8,
    'tracks' : tracks,
}


def TestAdapter():
    return adapter
########################################


class AdapterBase:
    
    """
    Shared base for Gant and Stress adapter
    """
    
    security = ClassSecurityInfo()


    security.declarePrivate('getAdapterStartDate')
    def getAdapterStartDate(self):
        "Returns the startdate of the view"
        return self.getStart()

    security.declarePrivate('getAdapterEndDate')
    def getAdapterEndDate(self):
        "Returns the startdate of the view"
        return self.getEnd()

    security.declarePrivate('getAdapterDateRange')
    def getAdapterDateRange(self, start, end):
        "Returns the range of dates between start and end"
        # normalize to start of day
        start = start.earliestTime()
        end = end.earliestTime()
        n_days = int(end - start)
        daterange = []
        for i in range(n_days):
            daterange.append(start)
            start += 1
        return daterange

    security.declarePrivate('getTracksInRange')
    def getTracksInRange(self, start, end):
        """
        Returns tracks inside range.
        !!!Overwrite in mixin class
        """
        return []

    def getTimeFormats(self):
        "Returns the sites date formats"
        properties = self.portal_properties.site_properties
        timeFormats = {}
        timeFormats['localTimeFormat'] = properties.localTimeFormat
        timeFormats['localLongTimeFormat'] = properties.localLongTimeFormat
        return timeFormats
        
    def getPortalYearRange(self):
        "Returns the sites year range"
        properties = self.portal_properties.site_properties
        yearRange = {}
        starting_year = properties.calendar_starting_year
        yearRange['portal_startyear'] = starting_year
        future_years = properties.calendar_future_years_available
        yearRange['portal_endyear'] = self.ZopeTime().year() + future_years
        return yearRange


    def absolute_time_url(self, obj):
        "Returns the absolute url with daterange for object"
        time_params = []
        a = time_params.append
        start = obj.getStart()
        a('start_year:int=%s' % start.year())
        a('start_month:int=%s' % start.month())
        a('start_day:int=%s' % start.day())
        end = obj.getEnd()
        a('end_year:int=%s' % end.year())
        a('end_month:int=%s' % end.month())
        a('end_day:int=%s' % end.day())
        return '%s?%s' % (obj.absolute_url(), '&'.join(time_params))
        

class GantAdapter(AdapterBase):

    """
    A mixin class for making gant adapters
    """
    
    security = ClassSecurityInfo()
    

    security.declarePublic('getGantAdapter')
    def getGantAdapter(self):
        """
        Returns an adapter (a complex dict) for rendering 
        the gant view of the project.
        """
        adapter = {'day_size' : 8}
        adapter.update(self.getTimeFormats())
        start_date = self.getAdapterStartDate()
        adapter['start_date'] = start_date
        end_date = self.getAdapterEndDate()
        adapter['end_date'] = end_date
        daterange = self.getAdapterDateRange(start_date, end_date)
        adapter['daterange'] = daterange
        adapter.update(self.getPortalYearRange())
        adapter['pre_size'] = 0
        # show the project itself as a track
        adapter['projectTrack'] = {
            'title':self.title_or_id(),
            'absolute_url' :  self.absolute_url(),
            'startdate' : self.getStart(),
            'enddate' : self.getEnd(),
            'physicalPath' : '/'.join(self.getPhysicalPath()),
            'id' : self.getId(),
        }
        adapter['projectTrack'].update(TrackCalc(self.getStart(), self.getEnd(), start_date, end_date).resultDict())
        # generate the tracks
        tracks = []
        # return sorted by enddate
        stones = self.getTracksInRange(start_date, end_date)
        decorated = [(s.getEnd(), s) for s in stones]
        decorated.sort()
        for stone in [d[-1] for d in decorated]:
            start, end = stone.getStart(), stone.getEnd()
            track = {
                'title' : stone.title_or_id(),
                'absolute_url' : stone.absolute_url(),
                'absolute_time_url' : self.absolute_time_url(stone),
                'startdate' : start,
                'enddate' : end,
                'physicalPath' : '/'.join(stone.getPhysicalPath()),
            }
            if stone.aq_parent == self:
                track['id'] = stone.getId()
            else:
                track['id'] = None
            track.update(TrackCalc(start, end, start_date, end_date).resultDict())
            tracks.append(track)
        adapter['tracks'] = tracks
        return adapter






if __name__ == '__main__':

    ga = GantAdapter()
    print ga.getAdapter()
