# -*- coding: ISO-8859-1 -*-

from Products.CMFCore.utils import getToolByName
from math import ceil

NUMBER = '%.2f'


def drdToDates(context, drd):
    start = context.ZopeTime(drd['start_year'], drd['start_month'], drd['start_day'])
    end = context.ZopeTime(drd['end_year'], drd['end_month'], drd['end_day'])
    return start, end


class HourAdapter:

    """
    Adapts projectmanager, projects and milestones to get hours overview
    """
    
    __allow_access_to_unprotected_subobjects__ = 1
    
    def __init__(self, context):
        self.context = context

    def __getattr__(self, attr):
        "attr magic"
        if attr[-2:] == '_F':
            return NUMBER % getattr(self, attr[:-2])()


    def tasks(self, status=None, start=None, end=None):
        """
        Returns tasks
        status in set( ('Needs action', 'Completed', 'In-Process', 'Cancelled',) )
        """
        return self.context.getTasks(status=status, start=start, end=end)

#    def activeTasks(self):
#        """
#        Returns tasks with
#        status in set( ('Needs action', 'In-Process',) )
#        """
#        context = self.context
#        drd = context.getDateRangeDict()
#        if drd['filter_on']:
#            start, end = drdToDates(context, drd)
#            return self.tasks(status=('Needs action', 'In-Process'), start=start, end=end)
#        else:
#            return self.tasks(status=('Needs action', 'In-Process',))
        
        
    def activeTasks(self, start=None, end=None):
        """
        Returns tasks with
        status in set( ('Needs action', 'In-Process',) )
        """
        return self.tasks(status=('Needs action', 'In-Process',) , start=start, end=end)
        
        
    ######
    
    def getAllocatedTime(self):
        """
        Returns number of hours allocated for tasks
        """
        return sum([t.getAllocatedTime() for t in self.tasks()])

#    def getUsedTime(self):
#        "Returns hours for scheduled tasks that are not finished"
#        return sum([t.getUsedTime() for t in self.activeTasks()])

    def getUsedTime(self):
        """
        Returns hours for scheduled tasks that are not finished
        """
        context = self.context
        drd = context.getDateRangeDict()
        if drd['filter_on']:
            start, end = drdToDates(context, drd)
        else:
            start = end = None
        tasks = self.tasks()
        return sum([t.getUsedTime(start, end) for t in tasks])
        
        
    def getRemainingTime(self):
        """
        Returns remaining time available
        """
        return sum([t.getRemainingTime() for t in self.tasks()])

    def getEstimatedRemainingTime(self):
        "Return estimated remaining time needed"
        return sum([t.getEstimatedRemainingTime() for t in self.tasks()])

    def getDifference(self):
        "Return estimated remaining time needed"
        return self.getRemainingTime() - self.getEstimatedRemainingTime()
        
        
    ######

    def taskTable(self):
        context = self.context
        drd = context.getDateRangeDict()
        if drd['filter_on']:
            start, end = drdToDates(context, drd)
        else:
            start = end = None
        table = []
        for task in self.tasks():
            estimatedRemainingTime = task.getEstimatedRemainingTime()
            remainingTime = task.getRemainingTime()
            difference = remainingTime - estimatedRemainingTime
            taskDict = {
                'title_or_id':task.title_or_id(),
                'absolute_url':task.absolute_url(),
                'getAllocatedTime':NUMBER % task.getAllocatedTime(),
                'nAttendees':task.nAttendees(),
                'getAllocatedTime':NUMBER % task.getAllocatedTime(),
                'getUsedTime':NUMBER % task.getUsedTime(start=start, end=end),
                'getRemainingTime':NUMBER % remainingTime,
                'getEstimatedRemainingTime': NUMBER % estimatedRemainingTime,
                'difference':difference,
            }
            table.append(taskDict)
        return table


    def title_or_id(self):
        return self.context.title_or_id()
        