# -*- coding: ISO-8859-1 -*-

# Copyright (c) 2005 by Max M
# Generator: ArchGenXML Version 1.4 devel 4 http://sf.net/projects/archetypes/
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

__author__  = '''Max M <maxm@mxm.dk>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo, Permissions
from GantAdapter import GantAdapter
from Products.Archetypes.atapi import *
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.mxmProjectManager.config import *
from RangeFilter import RangeFilter
from StressAdapter import StressAdapter
from HourAdapter import HourAdapter
from EventMover import EventMover
from TimeRegistrationMixin import TimeRegistrationMixin
from Products.Archetypes.public import DisplayList
try: # 2.3 and 2.4+ compatible set import
    set
except NameError:  
    from sets import Set as set

schema= Schema((
    StringField('title',
        widget=StringWidget(
            label='Title',
            label_msgid='mxmProjectManager_label_title',
            description='Enter a value for title.',
            description_msgid='mxmProjectManager_help_title',
            i18n_domain='mxmprojectmanager',
        ),
        accessor="Title",
    ),
    
    TextField('description',
        widget=TextAreaWidget(
            label='Description',
            label_msgid='mxmProjectManager_label_description',
            description='Enter a value for description.',
            description_msgid='mxmProjectManager_help_description',
            i18n_domain='mxmprojectmanager',
        ),
        accessor="Description",
    ),
    
    DateTimeField('start',
        widget=CalendarWidget(
            label='Start',
            label_msgid='mxmProjectManager_label_start',
            description='Enter a value for start.',
            description_msgid='mxmProjectManager_help_start',
            i18n_domain='mxmprojectmanager',
        ),
    ),
    
    DateTimeField('end',
        widget=CalendarWidget(
            label='End',
            label_msgid='mxmProjectManager_label_end',
            description='Enter a value for end.',
            description_msgid='mxmProjectManager_help_end',
            i18n_domain='mxmprojectmanager',
        ),
    ),
    
    ReferenceField('dependee',
        widget=SelectionWidget(
            label='Dependee',
            label_msgid='mxmProjectManager_label_dependee',
            description='Select a milestone that must be finished before this one can start.',
            description_msgid='mxmProjectManager_help_dependee',
            i18n_domain='mxmprojectmanager',
        ),
        relationship='dependent_of',
        vocabulary='milestonesDisplaylist',
    ),
    
),
)

class Milestone(BaseFolder, GantAdapter, TimeRegistrationMixin):
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseFolder,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name             = 'Milestone'

    portal_type = meta_type    = 'Milestone' 
    allowed_content_types      = ['mxmTodo']
    filter_content_types       = 1
    global_allow               = 0
    allow_discussion           = 0
    content_icon               = 'mxmMilestone.gif'
    immediate_view             = 'mxmProject_milestone_view'
    default_view               = 'mxmProject_milestone_view'
    typeDescription            = "Milestone"
    typeDescMsgId              = 'description_edit_milestone'

    schema = BaseFolderSchema + \
             schema

    _at_rename_after_creation = True

    actions = (
        {
            'id': 'view', 
            'name': 'View', 
            'action': 'string:${object_url}/mxmProject_milestone_view', 
            'permissions': (View,)
        }, 
        {
            'id': 'hours', 
            'name': 'Hours', 
            'action': 'string:${object_url}/mxmProjectManager_hours', 
            'permissions': (View,)
        }, 
        {
            'id': 'timereg', 
            'name': 'Time Registration',
            'action': 'string:${object_url}/mxmProjectManager_timereg', 
            'permissions': (View,)
        }, 
    )

    security.declareProtected(View, 'getMilestone')
    def getMilestone(self):
        "Return self"
        return self


    # overwrite the archetypes edit method
    security.declareProtected(ModifyPortalContent, 'processForm')
    def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
        BaseFolder.processForm(self, data=data, metadata=metadata,
                                REQUEST=REQUEST, values=values)
        # limit start-end times to project time
        if hasattr(self, 'getProject'):
            project = self.getProject()
            pStart = project.getStart()
            pEnd = project.getEnd()
            if self.getStart() < pStart: self.setStart(pStart)
            if self.getEnd() < pStart: self.setEnd(pStart)
            if self.getStart() > pEnd: self.setStart(pEnd)
            if self.getEnd() > pEnd: self.setEnd(pEnd)


    security.declareProtected(View, 'getStart')
    def getStart(self):
        "Returns start date-time"
        start = self.start
        if start:
            return start
        else:
            return self.aq_parent.getEnd()

    security.declareProtected(View, 'end')
    def getEnd(self):
        "Returns start date-time"
        end = self.end
        if end:
            return end
        else:
            return self.aq_parent.getEnd()

    security.declarePublic('actualStart')
    def actualStart(self):
        """
        
        """
        
        pass

    security.declarePublic('actualEnd')
    def actualEnd(self):
        """
        
        """
        
        pass

    # controller methods
    
    security.declareProtected(View, 'milestonesDisplaylist')
    def milestonesDisplaylist(self):
        "Returns a milestone displaylist"
        milestones = self.getProject().getMilestones()
        uids = [(str(m.UID()), m.title_or_id()) for m in milestones if m != self]
        return DisplayList([('','---')] + uids)


    # task getters

    security.declarePrivate('_getTasks')
    def _getTasks(self):
        """
        Returns tasks/todos in milestone.
        """
        return self.objectValues('mxmTodo')


    security.declareProtected(View, 'getTasks')
    def getTasks(self, status=None, start=None, end=None):
        """
        Returns tasks/todos in status
        status in set( ('Needs action', 'Completed', 'In-Process', 'Cancelled',) )
        """
        tasks = self._getTasks()
        # filter on range
        if start and end:
            tasks = RangeFilter(start, end, tasks).getInRange()
        # return all
        if status is None:
            return tasks
        else:
            status = set(status)
            result = []
            for task in tasks:
                if task.getStatus() in status:
                    result.append(task)
            return result


    security.declareProtected(Permissions.delete_objects, 'deleteTracks')
    def deleteTracks(self, ids=None, REQUEST=None):
        "Delete a list of tracks. Notice ... ids is really paths"
        if ids is None:
            ids = []
        if ids:
            self.manage_delObjects(ids)
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect(self.absolute_url())


    # range based getters


    security.declarePrivate('getTracksInRange')
    def getTracksInRange(self, start, end):
        """
        Returns tracks in range.
        """
        return self.getTasks(start=start, end=end)


    # view methods


    def getStressAdapter(self):
        "Returns a stress adapter for viewing a stress table"
        return StressAdapter(self).getStressAdapter()

    def getHourAdapter(self):
        "Returns an hour view adapter"
        return HourAdapter(self)


    security.declareProtected(View, 'view_mxmProjectManager_hours')
    def view_mxmProjectManager_hours(self):
        return HourAdapter(self)

    # arrangment of milestones

    def moveAbsolute(self, new_start):
        """
        Moves the milestone and its events to new absolute start time,
        keeping duration
        """
        em = EventMover(self)
        delta = em.getDelta(new_start)
        em.moveAbsolute(new_start)
        for task in self.getTasks():
            EventMover(task).moveRelative(delta)


    def moveRelative(self, delta):
        """
        Moves the event to new start time relative to current position,
        keeping duration
        """
        EventMover(self).moveRelative(delta)
        for task in self.getTasks():
            EventMover(task).moveRelative(delta)


    def getDependent(self):
        "Returns the milestones that are dependent of this one"
        # ugly and O^2 ...
        milestones = self.getProject().getMilestones()
        dependent = []
        for milestone in milestones:
            if milestone.getDependee() == self:
                dependent.append(milestone)
        return dependent


    def orderMilestones(self):
        "orders dependent milestones"
        for milestone in self.getDependent():
            milestone.moveAbsolute(self.getEnd())
            milestone.orderMilestones()


#    security.declareProtected(View, 'getResolvedRegistrations')
#    def getResolvedRegistrations(self):
#        "Fake method stub to minimize template logic."
#        return []


registerType(Milestone,PROJECTNAME)

