# -*- coding: ISO-8859-1 -*-

# Copyright (c) 2005 by Max M
# Generator: ArchGenXML Version 1.4 devel 4 http://sf.net/projects/archetypes/
# 
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

__author__  = '''Max M <maxm@mxm.dk>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo, Permissions
from DateTime import DateTime
from GantAdapter import GantAdapter
from Products.Archetypes.atapi import *
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.mxmProjectManager.config import *
from RangeFilter import RangeFilter
from HourAdapter import HourAdapter
from StressAdapter import StressAdapter
from TimeRegistrationMixin import TimeRegistrationMixin
try: # 2.3 and 2.4+ compatible set import
    set
except NameError:  
    from sets import Set as set

PROJECT_STATES = ('Tentative', 'In-Process','Needs action','Completed','Cancelled',)

schema= Schema((
    StringField('title', 
        widget=StringWidget(
            label='Title', 
            label_msgid='mxmProjectManager_label_title', 
            description='Enter a value for title.', 
            description_msgid='mxmProjectManager_help_title', 
            i18n_domain='mxmprojectmanager', 
        ), 
    ), 
    
    TextField('description', 
        widget=TextAreaWidget(
            label='Description', 
            label_msgid='mxmProjectManager_label_description', 
            description='Enter a value for description.', 
            description_msgid='mxmProjectManager_help_description', 
            i18n_domain='mxmprojectmanager', 
        ), 
    ), 
    
    DateTimeField('start', 
        widget=CalendarWidget(
            label='Start', 
            label_msgid='mxmProjectManager_label_start', 
            description='Enter a value for start.', 
            description_msgid='mxmProjectManager_help_start', 
            i18n_domain='mxmprojectmanager', 
        ), 
    ), 
    
    DateTimeField('end', 
        widget=CalendarWidget(
            label='End', 
            label_msgid='mxmProjectManager_label_end', 
            description='Enter a value for end.', 
            description_msgid='mxmProjectManager_help_end', 
            i18n_domain='mxmprojectmanager', 
        ), 
    ), 
    
    TextField('body', 
        widget=VisualWidget(
            label='Body', 
            label_msgid='mxmProjectManager_label_body', 
            description='Enter a value for body.', 
            description_msgid='mxmProjectManager_help_body', 
            i18n_domain='mxmprojectmanager', 
        ), 
    ), 
    
    StringField('state', 
        widget=SelectionWidget(
            label='State', 
            label_msgid='mxmProjectManager_label_state', 
            description='Enter a value for state.', 
            description_msgid='mxmProjectManager_help_state', 
            i18n_domain='mxmprojectmanager', 
        ),
        vocabulary=PROJECT_STATES,
        default=PROJECT_STATES[0],
    ),
    
    
 ), 
 )


class Project(BaseFolder, GantAdapter, TimeRegistrationMixin):
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseFolder, '__implements__', ()),)


    # This name appears in the 'add' box
    archetype_name             = 'Project'

    portal_type = meta_type    = 'Project' 
    allowed_content_types      = ['Milestone'] 
    filter_content_types       = 1
    global_allow               = 0
    allow_discussion           = 0
    content_icon               = 'mxmProject.gif'
    immediate_view             = 'mxmProject_view'
    default_view               = 'mxmProject_view'
    typeDescription            = "Project"
    typeDescMsgId              = 'description_edit_project'

    schema = BaseFolderSchema + \
             schema

    _at_rename_after_creation = True

    actions = (
        {
            'id': 'view', 
            'name': 'View', 
            'action': 'string:${object_url}/mxmProject_view', 
            'permissions': (View,)
        }, 
        {
            'id': 'hours', 
            'name': 'Hours', 
            'action': 'string:${object_url}/mxmProjectManager_hours', 
            'permissions': (View,)
        }, 
        {
            'id': 'timereg', 
            'name': 'Time Registration',
            'action': 'string:${object_url}/mxmProjectManager_timereg', 
            'permissions': (View,)
        }, 
    )

    security.declareProtected(View, 'getProject')
    def getProject(self):
        "Returns self"
        return self


    security.declareProtected(View, 'getStart')
    def getStart(self):
        "Returns start date-time"
        start = self.start
        if start:
            return start
        else:
            return self.aq_parent.getStart()

    security.declareProtected(View, 'getEnd')
    def getEnd(self):
        "Returns start date-time"
        end = self.end
        start = self.getStart()
        if end:
            return end
        else:
            return self.aq_parent.getEnd()

    # milestone getters

    security.declarePrivate('_getMilestones')
    def _getMilestones(self):
        """
        Returns the milestones in this project
        """
        milestones = self.objectValues('Milestone')
        return milestones

    security.declarePublic('getMilestones')
    def getMilestones(self, start=None, end=None):
        """
        Returns the milestones in this project
        """
        milestones = self._getMilestones()
        if start and end:
            return RangeFilter(start, end, milestones).getInRange()
        else:
            return milestones

    security.declareProtected(Permissions.delete_objects, 'deleteTracks')
    def deleteTracks(self, ids=None, REQUEST=None):
        "Delete a list of tracks. Notice ... ids is really paths"
        if ids is None:
            ids = []
        if ids:
            self.manage_delObjects(ids)
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    # Task getter

    security.declareProtected(View, 'getTasks')
    def getTasks(self, status=None, start=None, end=None):
        """
        Returns tasks/todos
        status in set( ('Needs action', 'Completed', 'In-Process', 'Cancelled',) )
        """
        result = []
        for milestone in self.getMilestones():
            result += milestone.getTasks(status, start, end)
        return result

    # Rescheduler


    security.declareProtected(ModifyPortalContent, 'orderMilestones')
    def orderMilestones(self):
        "Orders milestones"
        project_start = self.getStart()
        for milestone in self.getMilestones():
            if not milestone.getDependee():
                milestone.moveAbsolute(project_start)
                milestone.orderMilestones() # order dependent
        # redirecting back to gant view
        message = "Milestones have been ordered"
        redirect = self.REQUEST.RESPONSE.redirect
        redirect(self.absolute_url() + '?portal_status_message=%s' % message)

    # view methods

    security.declarePrivate('getTracksInRange')
    def getTracksInRange(self, start, end):
        """
        Returns tracks inside range.
        !!!Overwrite in mixin class
        """
        return self.getMilestones(start=start, end=end)


    security.declareProtected(View, 'view_mxmProjectManager_hours')
    def view_mxmProjectManager_hours(self):
        return HourAdapter(self)


    def getStressAdapter(self):
        "Returns a stress adapter for viewing a stress table"
        return StressAdapter(self).getStressAdapter()



registerType(Project, PROJECTNAME)
