# -*- coding: ISO-8859-1 -*-

from random import randint, random
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from TrackCalc import TrackCalc
try:
    set
except NameError:
    from sets import Set as set

#===============================================================================
#This is an example "interface" for a stressadapter.
#===============================================================================

daterange = []
date = DateTime()
for i in range(1,80):
    daterange.append(date)
    date += 1
n_dates = len(daterange)

day_max = 8.0
tracks = []
for i in range(20) :
    hours = [random()*14.0 for j in xrange(n_dates)]
    days = []
    for hour in hours:
        percent = (hour / day_max * 100.00)
        if percent <= 100.0:
            stresslevel = '%03i' % (int(percent/10) * 10)
        elif percent >= 130.0:
            stresslevel = 'alert2'
        else:
            stresslevel = 'alert1'
        day = {}
        day['hours'] = '%.2f' % hour
        day['stresslevel'] = stresslevel
        days.append(day)
    tracks.append({
        'title' : 'Track #%s' % i,
        'absolute_url' : '/stress?no=%s' % i,
        'days':days
    })

adapter = {
    'daterange' : daterange,
    'start_date': daterange[0],
    'end_date' : daterange[-1],
    'portal_startyear':2001,
    'portal_endyear':2010,
    'day_size' : 8,
    'tracks' : tracks,
}


def TestAdapter():
    return adapter

#===============================================================================
#This is the actual stress adapter
#===============================================================================

class StressAdapter:

    """
    Wraps objects
    """
    
    def __init__(self, obj):
        self.context = obj
        self.start = self.getAdapterStartDate()
        self.end = self.getAdapterEndDate()
        self.daterange = self.getAdapterDateRange(self.start, self.end)
        pm = obj.getProjectManager()
        self.daily_hours = {
            0 : pm.getSundayDefault(),
            1 : pm.getMondayDefault(),
            2 : pm.getTuesdayDefault(),
            3 : pm.getWednesdayDefault(),
            4 : pm.getThursdayDefault(),
            5 : pm.getFridayDefault(),
            6 : pm.getSaturdayDefault(),
        }

    def getRequestDate(self, name):
        "Returns the date from the request"
        req = self.context.REQUEST
        session = req.SESSION
        SES_YEAR = 'mxmproject_%s_year'
        SES_MONTH = 'mxmproject_%s_month'
        SES_DAY = 'mxmproject_%s_day'
        year = req.get('%s_year' % name, session.get(SES_YEAR % name, None))
        month = req.get('%s_month' % name, session.get(SES_MONTH % name, None))
        day = req.get('%s_day' % name, session.get(SES_DAY % name, None))
        if year and month and day:
            session[SES_YEAR % name] = year
            session[SES_MONTH % name] = month
            session[SES_DAY % name] = day
            return DateTime(year, month, day)
        else:
            return None

    def getAdapterStartDate(self):
        "Returns the startdate of the view"
        start = self.getRequestDate('start')
        if not start:
            start = self.context.getStart()
        return start

    def getAdapterEndDate(self):
        "Returns the startdate of the view"
        end = self.getRequestDate('end')
        if not end:
            end = self.context.getEnd()
        return end

    def getAdapterDateRange(self, start, end):
        "Returns the range of dates between start and end"
        n_days = int(end - start)
        daterange = []
        for i in range(n_days+2):
            daterange.append(start)
            start += 1
        return daterange

    def getPortalStartEndYears(self):
        "Returns the sites year range"
        properties = self.context.portal_properties.site_properties
        yearRange = {}
        starting_year = properties.calendar_starting_year
        yearRange['portal_startyear'] = starting_year
        future_years = properties.calendar_future_years_available
        yearRange['portal_endyear'] = self.context.ZopeTime().year() + future_years
        return yearRange

    def getTimeFormats(self):
        "Returns the sites date formats"
        properties = self.context.portal_properties.site_properties
        timeFormats = {}
        timeFormats['localTimeFormat'] = properties.localTimeFormat
        timeFormats['localLongTimeFormat'] = properties.localLongTimeFormat
        return timeFormats

    def eventInRange(self, event, start, end):
        "Returns true if event is in range"
        event_start = event.getStart()
        event_end = event.getEnd()
        return (
            (event_start and event_end)
            and (
                start <= event_start <= end
                or start <= event_end <= end
                or (event_start < start and event_end > end)
            )
        )



#    def getDailyHoursForEvent(self, event, date, taskDateRange):
#        "Returns the daily number of hours on a specific date for an event"
#        nonZeroTaskDates = [d for d in taskDateRange if self.daily_hours[d.dow()]]
#        if not nonZeroTaskDates and len(taskDateRange):
#            return event.getAllocatedTime() / len(taskDateRange)
#        if self.eventInRange(event, date.earliestTime(), date.latestTime()) and self.daily_hours[date.dow()]:
#            allocated = event.getAllocatedTime()
#            duration = len(nonZeroTaskDates)
#            if duration > 0.0:
#                return allocated / duration
#            else:
#                return 0.0
#        else:
#            return 0.0



    def getDailyHoursForEvent(self, event, date, nonZeroTaskDates):
        "Returns the daily number of hours on a specific date for an event"
        if self.eventInRange(event, date.earliestTime(), date.latestTime()) and self.daily_hours[date.dow()]:
            allocated = event.timePerAttendee()
            duration = len(nonZeroTaskDates)
            if duration > 0.0:
                return allocated / duration
            else:
                return 0.0
        else:
            return 0.0


    def getPercent(self, hours, date):
        day_max = self.daily_hours[date.dow()]
        if day_max == 0.0:
            if hours > 0.0: 
                return 300.0
            else:
                return 0.0
        return (hours / day_max * 100.00)

    def getStressLevel(self, hours, date):
        """
        Returns the stress level in percent.
        100% is the max amount of hours that date
        """
        percent = self.getPercent(hours, date)
        if percent <= 100.0:
            stresslevel = '%03i' % (int(percent/10) * 10)
        elif percent >= 120.0:
            stresslevel = 'alert2'
        else:
            stresslevel = 'alert1'
        return stresslevel


    def getTasksInRange(self):
        "returns taskt in range"
        return self.context.getTasks(start=self.start, end=self.end)


    def getNonZeroDates(self, dateRange):
        "Returns dates from daterange where alloted hours > 0.0"
        return [d for d in dateRange if self.daily_hours[d.dow()] > 0.0]
        

    def getMemberHours(self):
        """
        Returns a dict {memberId:[3.5, 12.3, 7.5]
        with the members daily hours in the range.
        """
        daterange = self.daterange
        n_dates = len(self.daterange)
        hourTable = {}
        for task in self.getTasksInRange():
            memberIds = task.attendee_ids()
            # find dates that does not have 0 hours. eg weekdays
            taskDateRange = self.getAdapterDateRange(task.getStart(), task.getEnd())
            nonZeroTaskDates = self.getNonZeroDates(taskDateRange)
            event_hours = [self.getDailyHoursForEvent(task, date, nonZeroTaskDates) for date in daterange]
            for id in memberIds:
                old_hours = hourTable.setdefault(id, [0.0]*n_dates)
                for i in range(n_dates):
                    old_hours[i] += event_hours[i]
        return hourTable



    def getStressAdapter(self):
        """
        Returns an adapter (a complex dict) for rendering 
        the stress view of the project.
        """
        context = self.context
        mtool = getToolByName(context, 'portal_membership')
        portal_url = getToolByName(context, 'portal_url').getPortalObject().absolute_url()
        adapter = {}
        adapter['title'] = 'Workload overview'
        adapter['member_fullname'] = ''
        adapter.update(self.getTimeFormats())
        adapter['day_size'] = 8 # 8x8 pixel
        adapter['start_date'] = self.start
        adapter['end_date'] = self.end
        daterange = self.daterange
        adapter['daterange'] = daterange
        adapter.update(self.getPortalStartEndYears())
        start, end = context.getStart(), context.getEnd()
        adapter['projectTrack'] = {
            'title': context.title_or_id(),
            'absolute_url' :  context.absolute_url(),
            'startdate' : start,
            'enddate' : end,
            'physicalPath' : '/'.join(context.getPhysicalPath()),
            'id' : context.getId(),
            'days':[],
        }
        adapter['projectTrack'].update(TrackCalc(start, end, self.start, self.end).resultDict())

        memberHours = self.getMemberHours()
        memberIds = memberHours.keys()
        tracks = []
        for memberId in memberIds:
            hours = memberHours[memberId]
            # create days for the track
            days = []
            for i, date in enumerate(daterange):
                day = {
                    'date':date,
                    'hours': hours[i],
                    'stresslevel':self.getStressLevel(hours[i], date),
                }
                days.append(day)
            # now create track
            member = mtool.getMemberById(memberId)
            title = '???'
            if not member is None:
                title = member.getProperty('fullname')
            track = {
                'title' : title,
                'absolute_url' : '%s/mxmProjectManager_participant?memberId=%s' % (context.absolute_url(), memberId),
                'days':days
            }
            tracks.append(track)
        adapter['tracks'] = tracks
        return adapter

    ######################

    def getProjectsInRange(self):
        return self.context.getProjects(start=self.start, end=self.end)

    def getMemberId(self):
        return self.context.REQUEST.get('memberId', None)

    def getProjectHours(self):
        """
        Returns a list of tuples [ (project, [3.5, 12.3, 7.5]), ]
        with the members daily hours in the range.
        for member only.
        """
        memberId = self.getMemberId()
        daterange = self.daterange
        n_dates = len(daterange)
        projects = []
        projectHours = []
        for project in self.getProjectsInRange():
            hours = [0.0]*n_dates
            for task in project.getTasks():
                if memberId in task.attendee_ids():
                    nonZeroTaskDates = self.getNonZeroDates(self.getAdapterDateRange(task.getStart(), task.getEnd()))
                    for i, date in enumerate(daterange):
                        dailyHours = self.getDailyHoursForEvent(task, date, nonZeroTaskDates)
                        hours[i] += dailyHours
            # Only add projects with hours for the member
            if max(hours) > 0.0:
                projectHours.append( (project, hours) )
        return projectHours

    def getMemberFullname(self, memberId):
        "Returns fullname of member"
        context = self.context
        mtool = getToolByName(context, 'portal_membership')
        member = mtool.getMemberById(memberId)
        fullname = '???'
        if member:
            fullname = member.getProperty('fullname', fullname)
        return fullname


    def getMemberStressAdapter(self):
        """
        Returns an adapter (a complex dict) for rendering 
        the stress view of the project.
        """
        context = self.context
        mtool = getToolByName(context, 'portal_membership')
        portal_url = getToolByName(context, 'portal_url').getPortalObject().absolute_url()
        adapter = {}
        adapter['title'] = 'Workload for'
        adapter['member_fullname'] =  self.getMemberFullname(self.getMemberId())
        adapter.update(self.getTimeFormats())
        adapter['day_size'] = 8 # 8x8 pixel
        adapter['start_date'] = self.start
        adapter['end_date'] = self.end
        daterange = self.daterange
        adapter['daterange'] = daterange
        adapter.update(self.getPortalStartEndYears())
        start, end = context.getStart(), context.getEnd()
        adapter['projectTrack'] = {
            'title': context.title_or_id(), #
            'absolute_url' :  context.absolute_url(),
            'startdate' : start,
            'enddate' : end,
            'physicalPath' : '/'.join(context.getPhysicalPath()),
            'id' : context.getId(),
            'days':[],
        }
        adapter['projectTrack'].update(TrackCalc(start, end, self.start, self.end).resultDict())
        
        # add tracks
        tracks = []
        for project, hours in self.getProjectHours():
            # create days for the track
            days = []
            for i, date in enumerate(daterange):
                day = {
                    'date':date,
                    'hours': hours[i],
                    'stresslevel':self.getStressLevel(hours[i], date),
                }
                days.append(day)
            # now create track
            track = {
                'title' : project.title_or_id(),
                'absolute_url' : project.absolute_url(),
                'days':days
            }
            tracks.append(track)
        adapter['tracks'] = tracks
        return adapter




if __name__ == '__main__':

    ga = StressAdapter()
    print ga.getAdapter()
