def round_(val):
    return int(round(val))



class TrackCalc:
    
    """
    Calculate stuff for gant tracks
    """

    def __init__(self, start, end, track_start, track_end):
        # start end of this track
        self.start = start
        self.end = end
        # start end of the track range
        self.track_start = track_start
        self.track_end = track_end


    def trackStart(self):
        "Calculates the startday of the gant tracks"
        return self.track_start

    def trackEnd(self):
        "Calculates the end of the gant tracks"
        return self.track_end

    def trackDuration(self):
        "Returns the duration of the tracks"
        return round_(self.trackEnd() - self.trackStart())


    def trackStartRelative(self):
        "Calculates the startday of the gant tracks"
        return 0

    def trackEndRelative(self):
        "Calculates the end of the gant tracks"
        return self.trackDuration()

    def trackSpan(self):
        "Returns the duration of the tracks. Is the duration + 1"
        return self.trackDuration() + 1



    def getStart(self):
        "Returns the events start time"
        return self.start

    def getEnd(self):
        "Returns the events end time"
        return self.end

    def getDuration(self):
        "Returns the events duration."
        return round_(self.getEnd() - self.getStart())


    def getStartRelative(self):
        "Returns the events start time"
        return round_(self.getStart() - self.trackStart())

    def getEndRelative(self):
        "Returns the events end time"
        return round_(self.getEnd() - self.trackStart())


    def getSpan(self):
        "Returns the events duration."
        return self.getDuration() + 1


    def showStart(self):
        "Should start diamond be visible"
        isInside = self.trackStart() <= self.getStart() <= self.trackEnd()
        return isInside

    def showEnd(self):
        "Should end diamond be visible"
        isInside = self.trackStart() <= self.getEnd() <= self.trackEnd()
        return isInside and self.croppedSpan() > 1


    def croppedStart(self):
        "Returns the cropped start time"
        return max((self.trackStartRelative(), self.getStartRelative()))

    def croppedEnd(self):
        "Returns the cropped start time"
        return min((self.trackEndRelative(), self.getEndRelative()))

    def croppedDuration(self):
        return round_(self.croppedEnd() - self.croppedStart())
        
    def croppedSpan(self):
        return self.croppedDuration() + 1
        

    def showDuration(self):
        return self.croppedSpan() > 2


    def showPre(self):
        "Show pre-bar"
        return self.croppedStart() > 0



    def resultDict(self):
        "Returns the result as a dict"
        duration = self.croppedSpan()
        if self.showStart(): duration -= 1
        if self.showEnd(): duration -= 1
        result = {
            'show_pre' : self.showPre(),
            'show_begin' : self.showStart(),
            'show_duration' : self.showDuration(),
            'show_end' : self.showEnd(),
            'start' : int(self.croppedStart()),
            'duration' : int(duration),
            'end' : self.croppedEnd(),
        }
        return result
    
    


if __name__ == '__main__':

    def prTrack(self):
        print ''.join(['='] *self.trackSpan())


    def prResult(r):
        result = []
        if r['show_pre']:
            result += ['.'] * r['start']
        if r['show_begin']:
            result.append('s')
        if r['show_duration']:
            result += ['-'] * r['duration']
        if r['show_end']:
            result.append('e')
        print ''.join(result)
        print '\n\n\n'


    def pr(self):
        result = []
        a = result.append
        # day numbers
        a('start:%s, end:%s, track-start:%s, track-end:%s' % (self.start, self.end, self.track_start, self.track_end))
        a(''.join(['0123456789']*3))

        event_pre = ''.join([' '] * self.getStart())
        event = ''.join(['-'] * self.getSpan() )
        a(event_pre + event)

        track_pre = ''.join([' '] * self.trackStart())
        track = ''.join(['='] * self.trackSpan())
        a(track_pre + track)

        yn = {True:'y', False:'n'}
        start = [' '] * 30
        start[self.getStart()] = yn[bool(self.showStart())]
        a(''.join(start))
        end = [' '] * 30
        end[self.getEnd()] = yn[bool(self.showEnd())]
        a(''.join(end))
        print '\n'.join(result)
        
    
    # starts and ends inside
    start, end = 5,15
    trackstart, trackend = 0,20
    tc = TrackCalc(start, end, trackstart, trackend)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())
    
    # starts and ends outside
    tc = TrackCalc(5,25, 10,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts and ends same time
    tc = TrackCalc(0,20, 0,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts before and ends inside
    tc = TrackCalc(5,15, 10,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts inside and ends at same time
    tc = TrackCalc(10,10, 0,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts inside and ends after
    tc = TrackCalc(10,11, 0,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts inside and ends after
    tc = TrackCalc(10,12, 0,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

    # starts inside and ends after
    tc = TrackCalc(10,13, 0,20)
    pr(tc)
    prTrack(tc)
    prResult(tc.resultDict())

